@@ -23,15 +23,17 @@ import java.util.Optional;
 
 import org.apache.hadoop.conf.Configuration;
 
+import org.apache.xtable.annotations.Stable;
 import org.apache.xtable.conversion.TargetTable;
 import org.apache.xtable.model.InternalTable;
 import org.apache.xtable.model.metadata.TableSyncMetadata;
 import org.apache.xtable.model.schema.InternalPartitionField;
 import org.apache.xtable.model.schema.InternalSchema;
-import org.apache.xtable.model.storage.DataFilesDiff;
+import org.apache.xtable.model.storage.InternalFilesDiff;
 import org.apache.xtable.model.storage.PartitionFileGroup;
 
 /** A client that provides the major functionality for syncing changes to a target system. */
+@Stable
 public interface ConversionTarget {
 
   /**
@@ -68,9 +70,9 @@ public interface ConversionTarget {
    * Syncs the changes in files to the target system. This method is required to both add and remove
    * files.
    *
-   * @param dataFilesDiff the diff that needs to be synced
+   * @param internalFilesDiff the diff that needs to be synced
    */
-  void syncFilesForDiff(DataFilesDiff dataFilesDiff);
+  void syncFilesForDiff(InternalFilesDiff internalFilesDiff);
 
   /**
    * Starts the sync and performs any initialization required
@@ -90,4 +92,14 @@ public interface ConversionTarget {
 
   /** Initializes the client with provided configuration */
   void init(TargetTable targetTable, Configuration configuration);
+
+  /**
+   * Retrieves the commit identifier from the target table that corresponds to a given source table
+   * commit identifier
+   *
+   * @param sourceIdentifier the unique identifier of the source table commit
+   * @return an {@link Optional} containing the target commit identifier if a corresponding commit
+   *     exists, or an empty {@link Optional} if no match is found
+   */
+  Optional<String> getTargetCommitIdentifier(String sourceIdentifier);
 }
