@@ -18,6 +18,8 @@
  
 package org.apache.xtable.conversion;
 
+import static org.apache.xtable.conversion.ConversionUtils.convertToSourceTable;
+
 import java.io.IOException;
 import java.time.Instant;
 import java.util.Collection;
@@ -37,15 +39,20 @@ import lombok.extern.log4j.Log4j2;
 
 import org.apache.hadoop.conf.Configuration;
 
+import org.apache.xtable.catalog.CatalogConversionFactory;
 import org.apache.xtable.exception.ReadException;
 import org.apache.xtable.model.IncrementalTableChanges;
 import org.apache.xtable.model.InstantsForIncrementalSync;
 import org.apache.xtable.model.InternalSnapshot;
+import org.apache.xtable.model.catalog.CatalogTableIdentifier;
 import org.apache.xtable.model.metadata.TableSyncMetadata;
 import org.apache.xtable.model.sync.SyncMode;
 import org.apache.xtable.model.sync.SyncResult;
+import org.apache.xtable.model.sync.SyncStatusCode;
 import org.apache.xtable.spi.extractor.ConversionSource;
 import org.apache.xtable.spi.extractor.ExtractFromSource;
+import org.apache.xtable.spi.sync.CatalogSync;
+import org.apache.xtable.spi.sync.CatalogSyncClient;
 import org.apache.xtable.spi.sync.ConversionTarget;
 import org.apache.xtable.spi.sync.TableFormatSync;
 
@@ -64,10 +71,17 @@ import org.apache.xtable.spi.sync.TableFormatSync;
 public class ConversionController {
   private final Configuration conf;
   private final ConversionTargetFactory conversionTargetFactory;
+  private final CatalogConversionFactory catalogConversionFactory;
   private final TableFormatSync tableFormatSync;
+  private final CatalogSync catalogSync;
 
   public ConversionController(Configuration conf) {
-    this(conf, ConversionTargetFactory.getInstance(), TableFormatSync.getInstance());
+    this(
+        conf,
+        ConversionTargetFactory.getInstance(),
+        CatalogConversionFactory.getInstance(),
+        TableFormatSync.getInstance(),
+        CatalogSync.getInstance());
   }
 
   /**
@@ -89,71 +103,158 @@ public class ConversionController {
     try (ConversionSource<COMMIT> conversionSource =
         conversionSourceProvider.getConversionSourceInstance(config.getSourceTable())) {
       ExtractFromSource<COMMIT> source = ExtractFromSource.of(conversionSource);
-
-      Map<String, ConversionTarget> conversionTargetByFormat =
-          config.getTargetTables().stream()
-              .collect(
-                  Collectors.toMap(
-                      TargetTable::getFormatName,
-                      targetTable -> conversionTargetFactory.createForFormat(targetTable, conf)));
-      // State for each TableFormat
-      Map<String, Optional<TableSyncMetadata>> lastSyncMetadataByFormat =
-          conversionTargetByFormat.entrySet().stream()
-              .collect(
-                  Collectors.toMap(
-                      Map.Entry::getKey, entry -> entry.getValue().getTableMetadata()));
-      Map<String, ConversionTarget> formatsToSyncIncrementally =
-          getFormatsToSyncIncrementally(
-              config,
-              conversionTargetByFormat,
-              lastSyncMetadataByFormat,
-              source.getConversionSource());
-      Map<String, ConversionTarget> formatsToSyncBySnapshot =
-          conversionTargetByFormat.entrySet().stream()
-              .filter(entry -> !formatsToSyncIncrementally.containsKey(entry.getKey()))
-              .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
-      SyncResultForTableFormats syncResultForSnapshotSync =
-          formatsToSyncBySnapshot.isEmpty()
-              ? SyncResultForTableFormats.builder().build()
-              : syncSnapshot(formatsToSyncBySnapshot, source);
-      SyncResultForTableFormats syncResultForIncrementalSync =
-          formatsToSyncIncrementally.isEmpty()
-              ? SyncResultForTableFormats.builder().build()
-              : syncIncrementalChanges(
-                  formatsToSyncIncrementally, lastSyncMetadataByFormat, source);
-      Map<String, SyncResult> syncResultsMerged =
-          new HashMap<>(syncResultForIncrementalSync.getLastSyncResult());
-      syncResultsMerged.putAll(syncResultForSnapshotSync.getLastSyncResult());
-      String successfulSyncs =
-          getFormatsWithStatusCode(syncResultsMerged, SyncResult.SyncStatusCode.SUCCESS);
-      if (!successfulSyncs.isEmpty()) {
-        log.info("Sync is successful for the following formats {}", successfulSyncs);
-      }
-      String failedSyncs =
-          getFormatsWithStatusCode(syncResultsMerged, SyncResult.SyncStatusCode.ERROR);
-      if (!failedSyncs.isEmpty()) {
-        log.error("Sync failed for the following formats {}", failedSyncs);
-      }
-      return syncResultsMerged;
+      return syncTableFormats(config, source, config.getSyncMode());
     } catch (IOException ioException) {
       throw new ReadException("Failed to close source converter", ioException);
     }
   }
 
+  /**
+   * Synchronizes the source table in conversion config to multiple target catalogs. If the
+   * configuration for the target table uses a different table format, synchronizes the table format
+   * first before syncing it to target catalog
+   *
+   * @param config A per table level config containing source table, target tables, target catalogs
+   *     and syncMode.
+   * @param conversionSourceProvider A provider for the {@link ConversionSource} instance for each
+   *     tableFormat, {@link ConversionSourceProvider#init(Configuration)} must be called before
+   *     calling this method.
+   * @return Returns a map containing the table format, and it's sync result. Run sync for a table *
+   *     with the provided per table level configuration.
+   */
+  public Map<String, SyncResult> syncTableAcrossCatalogs(
+      ConversionConfig config, Map<String, ConversionSourceProvider> conversionSourceProvider) {
+    if (config.getTargetTables() == null || config.getTargetTables().isEmpty()) {
+      throw new IllegalArgumentException("Please provide at-least one format to sync");
+    }
+    try (ConversionSource conversionSource =
+        conversionSourceProvider
+            .get(config.getSourceTable().getFormatName())
+            .getConversionSourceInstance(config.getSourceTable())) {
+      ExtractFromSource source = ExtractFromSource.of(conversionSource);
+      Map<String, SyncResult> tableFormatSyncResults =
+          syncTableFormats(config, source, config.getSyncMode());
+      Map<String, SyncResult> catalogSyncResults = new HashMap<>();
+      for (TargetTable targetTable : config.getTargetTables()) {
+        Map<CatalogTableIdentifier, CatalogSyncClient> catalogSyncClients =
+            config.getTargetCatalogs().get(targetTable).stream()
+                .collect(
+                    Collectors.toMap(
+                        TargetCatalogConfig::getCatalogTableIdentifier,
+                        targetCatalog ->
+                            catalogConversionFactory.createCatalogSyncClient(
+                                targetCatalog.getCatalogConfig(),
+                                targetTable.getFormatName(),
+                                conf)));
+        catalogSyncResults.put(
+            targetTable.getFormatName(),
+            syncCatalogsForTargetTable(
+                targetTable,
+                catalogSyncClients,
+                conversionSourceProvider.get(targetTable.getFormatName())));
+      }
+      mergeSyncResults(tableFormatSyncResults, catalogSyncResults);
+      return tableFormatSyncResults;
+    } catch (IOException ioException) {
+      throw new ReadException("Failed to close source converter", ioException);
+    }
+  }
+
+  /**
+   * Synchronizes the given source table format metadata in ConversionConfig to multiple target
+   * formats.
+   *
+   * @param config A per table level config containing tableBasePath, partitionFieldSpecConfig,
+   *     targetTableFormats and syncMode.
+   * @param source An extractor class for {@link ConversionSource} and allows fetching current
+   *     snapshot or incremental table changes.
+   * @param syncMode sync mode is either FULL or INCREMENTAL.
+   * @return Returns a map containing the table format, and it's sync result.
+   */
+  private <COMMIT> Map<String, SyncResult> syncTableFormats(
+      ConversionConfig config, ExtractFromSource<COMMIT> source, SyncMode syncMode) {
+    Map<String, ConversionTarget> conversionTargetByFormat =
+        config.getTargetTables().stream()
+            .filter(
+                targetTable ->
+                    !targetTable.getFormatName().equals(config.getSourceTable().getFormatName()))
+            .collect(
+                Collectors.toMap(
+                    TargetTable::getFormatName,
+                    targetTable -> conversionTargetFactory.createForFormat(targetTable, conf)));
+
+    Map<String, Optional<TableSyncMetadata>> lastSyncMetadataByFormat =
+        conversionTargetByFormat.entrySet().stream()
+            .collect(
+                Collectors.toMap(Map.Entry::getKey, entry -> entry.getValue().getTableMetadata()));
+    Map<String, ConversionTarget> formatsToSyncIncrementally =
+        getFormatsToSyncIncrementally(
+            syncMode,
+            conversionTargetByFormat,
+            lastSyncMetadataByFormat,
+            source.getConversionSource());
+    Map<String, ConversionTarget> formatsToSyncBySnapshot =
+        conversionTargetByFormat.entrySet().stream()
+            .filter(entry -> !formatsToSyncIncrementally.containsKey(entry.getKey()))
+            .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
+    SyncResultForTableFormats syncResultForSnapshotSync =
+        formatsToSyncBySnapshot.isEmpty()
+            ? SyncResultForTableFormats.builder().build()
+            : syncSnapshot(formatsToSyncBySnapshot, source);
+    SyncResultForTableFormats syncResultForIncrementalSync =
+        formatsToSyncIncrementally.isEmpty()
+            ? SyncResultForTableFormats.builder().build()
+            : syncIncrementalChanges(formatsToSyncIncrementally, lastSyncMetadataByFormat, source);
+    Map<String, SyncResult> syncResultsMerged =
+        new HashMap<>(syncResultForIncrementalSync.getLastSyncResult());
+    syncResultsMerged.putAll(syncResultForSnapshotSync.getLastSyncResult());
+    String successfulSyncs = getFormatsWithStatusCode(syncResultsMerged, SyncStatusCode.SUCCESS);
+    if (!successfulSyncs.isEmpty()) {
+      log.info("Sync is successful for the following formats {}", successfulSyncs);
+    }
+    String failedSyncs = getFormatsWithStatusCode(syncResultsMerged, SyncStatusCode.ERROR);
+    if (!failedSyncs.isEmpty()) {
+      log.error("Sync failed for the following formats {}", failedSyncs);
+    }
+    return syncResultsMerged;
+  }
+
+  /**
+   * Synchronizes the target table to multiple target catalogs.
+   *
+   * @param targetTable target table that needs to synced.
+   * @param catalogSyncClients Collection of catalog sync clients along with their table identifiers
+   *     for each target catalog.
+   * @param conversionSourceProvider A provider for the {@link ConversionSource} instance for the
+   *     table format of targetTable.
+   */
+  private SyncResult syncCatalogsForTargetTable(
+      TargetTable targetTable,
+      Map<CatalogTableIdentifier, CatalogSyncClient> catalogSyncClients,
+      ConversionSourceProvider conversionSourceProvider) {
+    return catalogSync.syncTable(
+        catalogSyncClients,
+        // We get the latest state of InternalTable for TargetTable
+        // and then synchronize it to catalogSyncClients.
+        conversionSourceProvider
+            .getConversionSourceInstance(convertToSourceTable(targetTable))
+            .getCurrentTable());
+  }
+
   private static String getFormatsWithStatusCode(
-      Map<String, SyncResult> syncResultsMerged, SyncResult.SyncStatusCode statusCode) {
+      Map<String, SyncResult> syncResultsMerged, SyncStatusCode statusCode) {
     return syncResultsMerged.entrySet().stream()
-        .filter(entry -> entry.getValue().getStatus().getStatusCode() == statusCode)
+        .filter(entry -> entry.getValue().getTableFormatSyncStatus().getStatusCode() == statusCode)
         .map(Map.Entry::getKey)
         .collect(Collectors.joining(","));
   }
 
   private <COMMIT> Map<String, ConversionTarget> getFormatsToSyncIncrementally(
-      ConversionConfig conversionConfig,
+      SyncMode syncMode,
       Map<String, ConversionTarget> conversionTargetByFormat,
       Map<String, Optional<TableSyncMetadata>> lastSyncMetadataByFormat,
       ConversionSource<COMMIT> conversionSource) {
-    if (conversionConfig.getSyncMode() == SyncMode.FULL) {
+    if (syncMode == SyncMode.FULL) {
       // Full sync requested by config, hence no incremental sync.
       return Collections.emptyMap();
     }
@@ -250,11 +351,14 @@ public class ConversionController {
 
   private InstantsForIncrementalSync getMostOutOfSyncCommitAndPendingCommits(
       Map<ConversionTarget, TableSyncMetadata> lastSyncMetadataByFormat) {
-    Optional<Instant> mostOutOfSyncCommit =
+    Instant mostOutOfSyncCommit =
         lastSyncMetadataByFormat.values().stream()
             .map(TableSyncMetadata::getLastInstantSynced)
             .sorted()
-            .findFirst();
+            .findFirst()
+            .orElseThrow(
+                () ->
+                    new IllegalArgumentException("No existing commits found for incremental sync"));
     List<Instant> allPendingInstants =
         lastSyncMetadataByFormat.values().stream()
             .map(TableSyncMetadata::getInstantsToConsiderForNextSync)
@@ -263,11 +367,27 @@ public class ConversionController {
             .sorted()
             .collect(Collectors.toList());
     return InstantsForIncrementalSync.builder()
-        .lastSyncInstant(mostOutOfSyncCommit.get())
+        .lastSyncInstant(mostOutOfSyncCommit)
         .pendingCommits(allPendingInstants)
         .build();
   }
 
+  private void mergeSyncResults(
+      Map<String, SyncResult> syncResultsMerged, Map<String, SyncResult> catalogSyncResults) {
+    catalogSyncResults.forEach(
+        (tableFormat, catalogSyncResult) -> {
+          syncResultsMerged.computeIfPresent(
+              tableFormat,
+              (k, syncResult) ->
+                  syncResult.toBuilder()
+                      .syncDuration(
+                          syncResult.getSyncDuration().plus(catalogSyncResult.getSyncDuration()))
+                      .catalogSyncStatusList(catalogSyncResult.getCatalogSyncStatusList())
+                      .build());
+          syncResultsMerged.computeIfAbsent(tableFormat, k -> catalogSyncResult);
+        });
+  }
+
   @Value
   @Builder
   private static class SyncResultForTableFormats {
