@@ -29,17 +29,13 @@ import lombok.NoArgsConstructor;
 
 import org.apache.spark.sql.types.ArrayType;
 import org.apache.spark.sql.types.DataType;
-import org.apache.spark.sql.types.DataTypes;
 import org.apache.spark.sql.types.DecimalType;
 import org.apache.spark.sql.types.MapType;
 import org.apache.spark.sql.types.Metadata;
-import org.apache.spark.sql.types.MetadataBuilder;
-import org.apache.spark.sql.types.StructField;
 import org.apache.spark.sql.types.StructType;
 
 import org.apache.xtable.collectors.CustomCollectors;
 import org.apache.xtable.exception.NotSupportedException;
-import org.apache.xtable.exception.SchemaExtractorException;
 import org.apache.xtable.model.schema.InternalField;
 import org.apache.xtable.model.schema.InternalSchema;
 import org.apache.xtable.model.schema.InternalType;
@@ -60,98 +56,16 @@ import org.apache.xtable.schema.SchemaUtils;
 public class DeltaSchemaExtractor {
   private static final String DELTA_COLUMN_MAPPING_ID = "delta.columnMapping.id";
   private static final DeltaSchemaExtractor INSTANCE = new DeltaSchemaExtractor();
+  // Timestamps in Delta are microsecond precision by default
+  private static final Map<InternalSchema.MetadataKey, Object>
+      DEFAULT_TIMESTAMP_PRECISION_METADATA =
+          Collections.singletonMap(
+              InternalSchema.MetadataKey.TIMESTAMP_PRECISION, InternalSchema.MetadataValue.MICROS);
 
   public static DeltaSchemaExtractor getInstance() {
     return INSTANCE;
   }
 
-  public StructType fromInternalSchema(InternalSchema internalSchema) {
-    StructField[] fields =
-        internalSchema.getFields().stream()
-            .map(
-                field ->
-                    new StructField(
-                        field.getName(),
-                        convertFieldType(field),
-                        field.getSchema().isNullable(),
-                        getMetaData(field.getSchema().getDataType())))
-            .toArray(StructField[]::new);
-    return new StructType(fields);
-  }
-
-  private DataType convertFieldType(InternalField field) {
-    switch (field.getSchema().getDataType()) {
-      case ENUM:
-      case STRING:
-        return DataTypes.StringType;
-      case INT:
-        return DataTypes.IntegerType;
-      case LONG:
-      case TIMESTAMP_NTZ:
-        return DataTypes.LongType;
-      case BYTES:
-      case FIXED:
-      case UUID:
-        return DataTypes.BinaryType;
-      case BOOLEAN:
-        return DataTypes.BooleanType;
-      case FLOAT:
-        return DataTypes.FloatType;
-      case DATE:
-        return DataTypes.DateType;
-      case TIMESTAMP:
-        return DataTypes.TimestampType;
-      case DOUBLE:
-        return DataTypes.DoubleType;
-      case DECIMAL:
-        int precision =
-            (int) field.getSchema().getMetadata().get(InternalSchema.MetadataKey.DECIMAL_PRECISION);
-        int scale =
-            (int) field.getSchema().getMetadata().get(InternalSchema.MetadataKey.DECIMAL_SCALE);
-        return DataTypes.createDecimalType(precision, scale);
-      case RECORD:
-        return fromInternalSchema(field.getSchema());
-      case MAP:
-        InternalField key =
-            field.getSchema().getFields().stream()
-                .filter(
-                    mapField ->
-                        InternalField.Constants.MAP_KEY_FIELD_NAME.equals(mapField.getName()))
-                .findFirst()
-                .orElseThrow(() -> new SchemaExtractorException("Invalid map schema"));
-        InternalField value =
-            field.getSchema().getFields().stream()
-                .filter(
-                    mapField ->
-                        InternalField.Constants.MAP_VALUE_FIELD_NAME.equals(mapField.getName()))
-                .findFirst()
-                .orElseThrow(() -> new SchemaExtractorException("Invalid map schema"));
-        return DataTypes.createMapType(
-            convertFieldType(key), convertFieldType(value), value.getSchema().isNullable());
-      case LIST:
-        InternalField element =
-            field.getSchema().getFields().stream()
-                .filter(
-                    arrayField ->
-                        InternalField.Constants.ARRAY_ELEMENT_FIELD_NAME.equals(
-                            arrayField.getName()))
-                .findFirst()
-                .orElseThrow(() -> new SchemaExtractorException("Invalid array schema"));
-        return DataTypes.createArrayType(
-            convertFieldType(element), element.getSchema().isNullable());
-      default:
-        throw new NotSupportedException("Unsupported type: " + field.getSchema().getDataType());
-    }
-  }
-
-  private Metadata getMetaData(InternalType type) {
-    if (type == InternalType.UUID) {
-      return new MetadataBuilder().putString(InternalSchema.XTABLE_LOGICAL_TYPE, "uuid").build();
-    } else {
-      return Metadata.empty();
-    }
-  }
-
   public InternalSchema toInternalSchema(StructType structType) {
     return toInternalSchema(structType, null, false, null, null);
   }
@@ -201,11 +115,11 @@ public class DeltaSchemaExtractor {
         break;
       case "timestamp":
         type = InternalType.TIMESTAMP;
-        // Timestamps in Delta are microsecond precision by default
-        metadata =
-            Collections.singletonMap(
-                InternalSchema.MetadataKey.TIMESTAMP_PRECISION,
-                InternalSchema.MetadataValue.MICROS);
+        metadata = DEFAULT_TIMESTAMP_PRECISION_METADATA;
+        break;
+      case "timestamp_ntz":
+        type = InternalType.TIMESTAMP_NTZ;
+        metadata = DEFAULT_TIMESTAMP_PRECISION_METADATA;
         break;
       case "struct":
         StructType structType = (StructType) dataType;
