@@ -18,8 +18,12 @@
  
 package org.apache.xtable.conversion;
 
+import static org.apache.xtable.conversion.ConversionUtils.convertToSourceTable;
+import static org.apache.xtable.model.storage.TableFormat.DELTA;
 import static org.apache.xtable.model.storage.TableFormat.HUDI;
+import static org.apache.xtable.model.storage.TableFormat.ICEBERG;
 import static org.junit.jupiter.api.Assertions.*;
+import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.argThat;
 import static org.mockito.ArgumentMatchers.eq;
 import static org.mockito.Mockito.mock;
@@ -36,6 +40,7 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
+import java.util.function.Function;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
@@ -43,17 +48,23 @@ import org.apache.hadoop.conf.Configuration;
 import org.junit.jupiter.api.Test;
 import org.mockito.ArgumentMatcher;
 
+import com.google.common.collect.ImmutableMap;
+
+import org.apache.xtable.catalog.CatalogConversionFactory;
 import org.apache.xtable.model.CommitsBacklog;
 import org.apache.xtable.model.IncrementalTableChanges;
 import org.apache.xtable.model.InstantsForIncrementalSync;
 import org.apache.xtable.model.InternalSnapshot;
 import org.apache.xtable.model.InternalTable;
 import org.apache.xtable.model.TableChange;
+import org.apache.xtable.model.catalog.ThreePartHierarchicalTableIdentifier;
 import org.apache.xtable.model.metadata.TableSyncMetadata;
 import org.apache.xtable.model.storage.TableFormat;
 import org.apache.xtable.model.sync.SyncMode;
 import org.apache.xtable.model.sync.SyncResult;
 import org.apache.xtable.spi.extractor.ConversionSource;
+import org.apache.xtable.spi.sync.CatalogSync;
+import org.apache.xtable.spi.sync.CatalogSyncClient;
 import org.apache.xtable.spi.sync.ConversionTarget;
 import org.apache.xtable.spi.sync.TableFormatSync;
 
@@ -62,18 +73,28 @@ public class TestConversionController {
   private final Configuration mockConf = mock(Configuration.class);
   private final ConversionSourceProvider<Instant> mockConversionSourceProvider =
       mock(ConversionSourceProvider.class);
+  private final ConversionSourceProvider<Instant> mockConversionSourceProvider2 =
+      mock(ConversionSourceProvider.class);
+  private final ConversionSourceProvider<Instant> mockConversionSourceProvider3 =
+      mock(ConversionSourceProvider.class);
+
   private final ConversionSource<Instant> mockConversionSource = mock(ConversionSource.class);
   private final ConversionTargetFactory mockConversionTargetFactory =
       mock(ConversionTargetFactory.class);
+  private final CatalogConversionFactory mockCatalogConversionFactory =
+      mock(CatalogConversionFactory.class);
   private final TableFormatSync tableFormatSync = mock(TableFormatSync.class);
+  private final CatalogSync catalogSync = mock(CatalogSync.class);
   private final ConversionTarget mockConversionTarget1 = mock(ConversionTarget.class);
   private final ConversionTarget mockConversionTarget2 = mock(ConversionTarget.class);
+  private final CatalogSyncClient mockCatalogSyncClient1 = mock(CatalogSyncClient.class);
+  private final CatalogSyncClient mockCatalogSyncClient2 = mock(CatalogSyncClient.class);
 
   @Test
   void testAllSnapshotSyncAsPerConfig() {
     SyncMode syncMode = SyncMode.FULL;
     InternalTable internalTable = getInternalTable();
-    InternalSnapshot internalSnapshot = buildSnapshot(internalTable, "v1");
+    InternalSnapshot internalSnapshot = buildSnapshot(internalTable, "v1", "0");
     Instant instantBeforeHour = Instant.now().minus(Duration.ofHours(1));
     SyncResult syncResult = buildSyncResult(syncMode, instantBeforeHour);
     Map<String, SyncResult> perTableResults = new HashMap<>();
@@ -96,7 +117,12 @@ public class TestConversionController {
             eq(internalSnapshot)))
         .thenReturn(perTableResults);
     ConversionController conversionController =
-        new ConversionController(mockConf, mockConversionTargetFactory, tableFormatSync);
+        new ConversionController(
+            mockConf,
+            mockConversionTargetFactory,
+            mockCatalogConversionFactory,
+            tableFormatSync,
+            catalogSync);
     Map<String, SyncResult> result =
         conversionController.sync(conversionConfig, mockConversionSourceProvider);
     assertEquals(perTableResults, result);
@@ -146,16 +172,19 @@ public class TestConversionController {
     CommitsBacklog<Instant> commitsBacklog =
         CommitsBacklog.<Instant>builder().commitsToProcess(instantsToProcess).build();
     Optional<TableSyncMetadata> conversionTarget1Metadata =
-        Optional.of(TableSyncMetadata.of(icebergLastSyncInstant, pendingInstantsForIceberg));
+        Optional.of(
+            TableSyncMetadata.of(icebergLastSyncInstant, pendingInstantsForIceberg, "TEST", "0"));
     when(mockConversionTarget1.getTableMetadata()).thenReturn(conversionTarget1Metadata);
     Optional<TableSyncMetadata> conversionTarget2Metadata =
-        Optional.of(TableSyncMetadata.of(deltaLastSyncInstant, pendingInstantsForDelta));
+        Optional.of(
+            TableSyncMetadata.of(deltaLastSyncInstant, pendingInstantsForDelta, "TEST", "0"));
     when(mockConversionTarget2.getTableMetadata()).thenReturn(conversionTarget2Metadata);
     when(mockConversionSource.getCommitsBacklog(instantsForIncrementalSync))
         .thenReturn(commitsBacklog);
     List<TableChange> tableChanges = new ArrayList<>();
-    for (Instant instant : instantsToProcess) {
-      TableChange tableChange = getTableChange(instant);
+    for (int i = 0; i < instantsToProcess.size(); i++) {
+      Instant instant = instantsToProcess.get(i);
+      TableChange tableChange = getTableChange(instant, String.valueOf(i));
       tableChanges.add(tableChange);
       when(mockConversionSource.getTableChangeForCommit(instant)).thenReturn(tableChange);
     }
@@ -182,7 +211,12 @@ public class TestConversionController {
     expectedSyncResult.put(TableFormat.ICEBERG, getLastSyncResult(icebergSyncResults));
     expectedSyncResult.put(TableFormat.DELTA, getLastSyncResult(deltaSyncResults));
     ConversionController conversionController =
-        new ConversionController(mockConf, mockConversionTargetFactory, tableFormatSync);
+        new ConversionController(
+            mockConf,
+            mockConversionTargetFactory,
+            mockCatalogConversionFactory,
+            tableFormatSync,
+            catalogSync);
     Map<String, SyncResult> result =
         conversionController.sync(conversionConfig, mockConversionSourceProvider);
     assertEquals(expectedSyncResult, result);
@@ -193,7 +227,7 @@ public class TestConversionController {
     SyncMode syncMode = SyncMode.INCREMENTAL;
     InternalTable internalTable = getInternalTable();
     Instant instantBeforeHour = Instant.now().minus(Duration.ofHours(1));
-    InternalSnapshot internalSnapshot = buildSnapshot(internalTable, "v1");
+    InternalSnapshot internalSnapshot = buildSnapshot(internalTable, "v1", "0");
     SyncResult syncResult = buildSyncResult(syncMode, instantBeforeHour);
     Map<String, SyncResult> syncResults = new HashMap<>();
     syncResults.put(TableFormat.ICEBERG, syncResult);
@@ -216,9 +250,11 @@ public class TestConversionController {
 
     // Both Iceberg and Delta last synced at instantAt5 and have no pending instants.
     when(mockConversionTarget1.getTableMetadata())
-        .thenReturn(Optional.of(TableSyncMetadata.of(instantAt5, Collections.emptyList())));
+        .thenReturn(
+            Optional.of(TableSyncMetadata.of(instantAt5, Collections.emptyList(), "TEST", "0")));
     when(mockConversionTarget2.getTableMetadata())
-        .thenReturn(Optional.of(TableSyncMetadata.of(instantAt5, Collections.emptyList())));
+        .thenReturn(
+            Optional.of(TableSyncMetadata.of(instantAt5, Collections.emptyList(), "TEST", "0")));
 
     when(mockConversionSource.getCurrentSnapshot()).thenReturn(internalSnapshot);
     when(tableFormatSync.syncSnapshot(
@@ -226,7 +262,12 @@ public class TestConversionController {
             eq(internalSnapshot)))
         .thenReturn(syncResults);
     ConversionController conversionController =
-        new ConversionController(mockConf, mockConversionTargetFactory, tableFormatSync);
+        new ConversionController(
+            mockConf,
+            mockConversionTargetFactory,
+            mockCatalogConversionFactory,
+            tableFormatSync,
+            catalogSync);
     Map<String, SyncResult> result =
         conversionController.sync(conversionConfig, mockConversionSourceProvider);
     assertEquals(syncResults, result);
@@ -273,22 +314,26 @@ public class TestConversionController {
         CommitsBacklog.<Instant>builder().commitsToProcess(instantsToProcess).build();
     when(mockConversionTarget1.getTableMetadata())
         .thenReturn(
-            Optional.of(TableSyncMetadata.of(icebergLastSyncInstant, pendingInstantsForIceberg)));
+            Optional.of(
+                TableSyncMetadata.of(
+                    icebergLastSyncInstant, pendingInstantsForIceberg, "TEST", "0")));
     Optional<TableSyncMetadata> conversionTarget2Metadata =
-        Optional.of(TableSyncMetadata.of(deltaLastSyncInstant, pendingInstantsForDelta));
+        Optional.of(
+            TableSyncMetadata.of(deltaLastSyncInstant, pendingInstantsForDelta, "TEST", "0"));
     when(mockConversionTarget2.getTableMetadata()).thenReturn(conversionTarget2Metadata);
     when(mockConversionSource.getCommitsBacklog(instantsForIncrementalSync))
         .thenReturn(commitsBacklog);
     List<TableChange> tableChanges = new ArrayList<>();
-    for (Instant instant : instantsToProcess) {
-      TableChange tableChange = getTableChange(instant);
+    for (int i = 0; i < instantsToProcess.size(); i++) {
+      Instant instant = instantsToProcess.get(i);
+      TableChange tableChange = getTableChange(instant, String.valueOf(i));
       tableChanges.add(tableChange);
       when(mockConversionSource.getTableChangeForCommit(instant)).thenReturn(tableChange);
     }
     // Iceberg needs to sync by snapshot since instant15 is affected by table clean-up.
     InternalTable internalTable = getInternalTable();
     Instant instantBeforeHour = Instant.now().minus(Duration.ofHours(1));
-    InternalSnapshot internalSnapshot = buildSnapshot(internalTable, "v1");
+    InternalSnapshot internalSnapshot = buildSnapshot(internalTable, "v1", "0");
     SyncResult syncResult = buildSyncResult(syncMode, instantBeforeHour);
     Map<String, SyncResult> snapshotResult =
         Collections.singletonMap(TableFormat.ICEBERG, syncResult);
@@ -310,7 +355,12 @@ public class TestConversionController {
     expectedSyncResult.put(TableFormat.ICEBERG, syncResult);
     expectedSyncResult.put(TableFormat.DELTA, getLastSyncResult(deltaSyncResults));
     ConversionController conversionController =
-        new ConversionController(mockConf, mockConversionTargetFactory, tableFormatSync);
+        new ConversionController(
+            mockConf,
+            mockConversionTargetFactory,
+            mockCatalogConversionFactory,
+            tableFormatSync,
+            catalogSync);
     Map<String, SyncResult> result =
         conversionController.sync(conversionConfig, mockConversionSourceProvider);
     assertEquals(expectedSyncResult, result);
@@ -347,10 +397,12 @@ public class TestConversionController {
     CommitsBacklog<Instant> commitsBacklog =
         CommitsBacklog.<Instant>builder().commitsToProcess(instantsToProcess).build();
     Optional<TableSyncMetadata> conversionTarget1Metadata =
-        Optional.of(TableSyncMetadata.of(icebergLastSyncInstant, Collections.emptyList()));
+        Optional.of(
+            TableSyncMetadata.of(icebergLastSyncInstant, Collections.emptyList(), "TEST", "0"));
     when(mockConversionTarget1.getTableMetadata()).thenReturn(conversionTarget1Metadata);
     Optional<TableSyncMetadata> conversionTarget2Metadata =
-        Optional.of(TableSyncMetadata.of(deltaLastSyncInstant, Collections.emptyList()));
+        Optional.of(
+            TableSyncMetadata.of(deltaLastSyncInstant, Collections.emptyList(), "TEST", "0"));
     when(mockConversionTarget2.getTableMetadata()).thenReturn(conversionTarget2Metadata);
     when(mockConversionSource.getCommitsBacklog(instantsForIncrementalSync))
         .thenReturn(commitsBacklog);
@@ -368,36 +420,143 @@ public class TestConversionController {
     // Iceberg and Delta have no commits to sync
     Map<String, SyncResult> expectedSyncResult = Collections.emptyMap();
     ConversionController conversionController =
-        new ConversionController(mockConf, mockConversionTargetFactory, tableFormatSync);
+        new ConversionController(
+            mockConf,
+            mockConversionTargetFactory,
+            mockCatalogConversionFactory,
+            tableFormatSync,
+            catalogSync);
     Map<String, SyncResult> result =
         conversionController.sync(conversionConfig, mockConversionSourceProvider);
     assertEquals(expectedSyncResult, result);
   }
 
+  @Test
+  void testNoTableFormatConversionWithMultipleCatalogSync() {
+    SyncMode syncMode = SyncMode.INCREMENTAL;
+    List<TargetCatalogConfig> targetCatalogs =
+        Arrays.asList(getTargetCatalog("1"), getTargetCatalog("2"));
+    InternalTable internalTable = getInternalTable();
+    InternalSnapshot internalSnapshot = buildSnapshot(internalTable, "v1", "0");
+    // Conversion source and target mocks.
+    ConversionConfig conversionConfig =
+        getTableSyncConfig(
+            Arrays.asList(TableFormat.ICEBERG, TableFormat.DELTA), syncMode, targetCatalogs);
+    when(mockConversionSourceProvider.getConversionSourceInstance(
+            conversionConfig.getSourceTable()))
+        .thenReturn(mockConversionSource);
+    when(mockConversionSourceProvider.getConversionSourceInstance(
+            convertToSourceTable(conversionConfig.getTargetTables().get(0))))
+        .thenReturn(mockConversionSource);
+    when(mockConversionSourceProvider.getConversionSourceInstance(
+            convertToSourceTable(conversionConfig.getTargetTables().get(1))))
+        .thenReturn(mockConversionSource);
+    when(mockConversionTargetFactory.createForFormat(
+            conversionConfig.getTargetTables().get(0), mockConf))
+        .thenReturn(mockConversionTarget1);
+    when(mockConversionTargetFactory.createForFormat(
+            conversionConfig.getTargetTables().get(1), mockConf))
+        .thenReturn(mockConversionTarget2);
+    when(mockConversionSource.getCurrentSnapshot()).thenReturn(internalSnapshot);
+    when(mockConversionSource.getCurrentTable()).thenReturn(getInternalTable());
+    // Mocks for tableFormatSync.
+    Instant instantBeforeHour = Instant.now().minus(Duration.ofHours(1));
+    Instant syncStartTime = Instant.now();
+    SyncResult syncResult =
+        buildSyncResult(syncMode, instantBeforeHour, syncStartTime, Duration.ofSeconds(1));
+    Map<String, SyncResult> tableFormatSyncResults =
+        buildPerTableResult(Arrays.asList(ICEBERG, DELTA), syncResult);
+    when(tableFormatSync.syncSnapshot(
+            argThat(containsAll(Arrays.asList(mockConversionTarget1, mockConversionTarget2))),
+            eq(internalSnapshot)))
+        .thenReturn(tableFormatSyncResults);
+    // Mocks for catalogSync.
+    when(mockCatalogConversionFactory.createCatalogSyncClient(
+            eq(targetCatalogs.get(0).getCatalogConfig()), any(), eq(mockConf)))
+        .thenReturn(mockCatalogSyncClient1);
+    when(mockCatalogConversionFactory.createCatalogSyncClient(
+            eq(targetCatalogs.get(1).getCatalogConfig()), any(), eq(mockConf)))
+        .thenReturn(mockCatalogSyncClient2);
+    when(catalogSync.syncTable(
+            eq(
+                ImmutableMap.of(
+                    targetCatalogs.get(0).getCatalogTableIdentifier(), mockCatalogSyncClient1,
+                    targetCatalogs.get(1).getCatalogTableIdentifier(), mockCatalogSyncClient2)),
+            any()))
+        .thenReturn(
+            buildSyncResult(syncMode, syncStartTime, instantBeforeHour, Duration.ofSeconds(3)));
+    ConversionController conversionController =
+        new ConversionController(
+            mockConf,
+            mockConversionTargetFactory,
+            mockCatalogConversionFactory,
+            tableFormatSync,
+            catalogSync);
+    // Mocks for conversionSourceProviders.
+    Map<String, ConversionSourceProvider> conversionSourceProviders = new HashMap<>();
+    conversionSourceProviders.put(HUDI, mockConversionSourceProvider);
+    conversionSourceProviders.put(ICEBERG, mockConversionSourceProvider);
+    conversionSourceProviders.put(DELTA, mockConversionSourceProvider);
+    // Assert results.
+    Map<String, SyncResult> mergedSyncResults =
+        buildPerTableResult(
+            Arrays.asList(ICEBERG, DELTA),
+            syncResult.toBuilder().syncDuration(Duration.ofSeconds(4)).build());
+    Map<String, SyncResult> result =
+        conversionController.syncTableAcrossCatalogs(conversionConfig, conversionSourceProviders);
+    assertEquals(mergedSyncResults, result);
+  }
+
   private SyncResult getLastSyncResult(List<SyncResult> syncResults) {
     return syncResults.get(syncResults.size() - 1);
   }
 
+  private Map<String, SyncResult> buildPerTableResult(
+      List<String> tableFormats, SyncResult syncResult) {
+    Map<String, SyncResult> perTableResults = new HashMap<>();
+    tableFormats.forEach(tableFormat -> perTableResults.put(tableFormat, syncResult));
+    return perTableResults;
+  }
+
   private List<SyncResult> buildSyncResults(List<Instant> instantList) {
     return instantList.stream()
         .map(instant -> buildSyncResult(SyncMode.INCREMENTAL, instant))
         .collect(Collectors.toList());
   }
 
-  private TableChange getTableChange(Instant instant) {
-    return TableChange.builder().tableAsOfChange(getInternalTable(instant)).build();
+  private TableChange getTableChange(Instant instant, String sourceIdentifier) {
+    return TableChange.builder()
+        .tableAsOfChange(getInternalTable(instant))
+        .sourceIdentifier(sourceIdentifier)
+        .build();
   }
 
   private SyncResult buildSyncResult(SyncMode syncMode, Instant lastSyncedInstant) {
     return SyncResult.builder()
         .mode(syncMode)
         .lastInstantSynced(lastSyncedInstant)
-        .status(SyncResult.SyncStatus.SUCCESS)
+        .tableFormatSyncStatus(SyncResult.SyncStatus.SUCCESS)
         .build();
   }
 
-  private InternalSnapshot buildSnapshot(InternalTable internalTable, String version) {
-    return InternalSnapshot.builder().table(internalTable).version(version).build();
+  private SyncResult buildSyncResult(
+      SyncMode syncMode, Instant syncStartTime, Instant lastSyncedInstant, Duration duration) {
+    return SyncResult.builder()
+        .mode(syncMode)
+        .lastInstantSynced(lastSyncedInstant)
+        .syncStartTime(syncStartTime)
+        .syncDuration(duration)
+        .tableFormatSyncStatus(SyncResult.SyncStatus.SUCCESS)
+        .build();
+  }
+
+  private InternalSnapshot buildSnapshot(
+      InternalTable internalTable, String version, String sourceIdentifier) {
+    return InternalSnapshot.builder()
+        .table(internalTable)
+        .version(version)
+        .sourceIdentifier(sourceIdentifier)
+        .build();
   }
 
   private InternalTable getInternalTable() {
@@ -413,6 +572,13 @@ public class TestConversionController {
   }
 
   private ConversionConfig getTableSyncConfig(List<String> targetTableFormats, SyncMode syncMode) {
+    return getTableSyncConfig(targetTableFormats, syncMode, Collections.emptyList());
+  }
+
+  private ConversionConfig getTableSyncConfig(
+      List<String> targetTableFormats,
+      SyncMode syncMode,
+      List<TargetCatalogConfig> targetCatalogs) {
     SourceTable sourceTable =
         SourceTable.builder()
             .name("tablename")
@@ -434,10 +600,27 @@ public class TestConversionController {
     return ConversionConfig.builder()
         .sourceTable(sourceTable)
         .targetTables(targetTables)
+        .targetCatalogs(
+            targetTables.stream()
+                .collect(Collectors.toMap(Function.identity(), k -> targetCatalogs)))
         .syncMode(syncMode)
         .build();
   }
 
+  private TargetCatalogConfig getTargetCatalog(String suffix) {
+    return TargetCatalogConfig.builder()
+        .catalogConfig(
+            ExternalCatalogConfig.builder()
+                .catalogId("catalogId-" + suffix)
+                .catalogSyncClientImpl("catalogImpl-" + suffix)
+                .catalogProperties(Collections.emptyMap())
+                .build())
+        .catalogTableIdentifier(
+            new ThreePartHierarchicalTableIdentifier(
+                "target-database" + suffix, "target-tableName" + suffix))
+        .build();
+  }
+
   private static <T> ArgumentMatcher<Collection<T>> containsAll(Collection<T> expected) {
     return actual -> actual.size() == expected.size() && actual.containsAll(expected);
   }
