@@ -56,6 +56,7 @@ public class TestDeltaSchemaExtractor {
                                 .name("boolean")
                                 .dataType(InternalType.BOOLEAN)
                                 .isNullable(false)
+                                .comment("requiredBooleanComment")
                                 .build())
                         .build(),
                     InternalField.builder()
@@ -226,7 +227,7 @@ public class TestDeltaSchemaExtractor {
 
     StructType structRepresentation =
         new StructType()
-            .add("requiredBoolean", DataTypes.BooleanType, false)
+            .add("requiredBoolean", DataTypes.BooleanType, false, "requiredBooleanComment")
             .add("optionalBoolean", DataTypes.BooleanType, true)
             .add("requiredInt", DataTypes.IntegerType, false)
             .add("optionalInt", DataTypes.IntegerType, true)
@@ -245,43 +246,12 @@ public class TestDeltaSchemaExtractor {
             .add("requiredDecimal", DataTypes.createDecimalType(10, 2), false)
             .add("optionalDecimal", DataTypes.createDecimalType(10, 2), true);
 
-    Assertions.assertEquals(
-        structRepresentation,
-        DeltaSchemaExtractor.getInstance().fromInternalSchema(internalSchema));
     Assertions.assertEquals(
         internalSchema, DeltaSchemaExtractor.getInstance().toInternalSchema(structRepresentation));
   }
 
   @Test
   public void testFixedBytes() {
-    InternalSchema internalSchemaOriginal =
-        InternalSchema.builder()
-            .name("struct")
-            .dataType(InternalType.RECORD)
-            .isNullable(false)
-            .fields(
-                Arrays.asList(
-                    InternalField.builder()
-                        .name("requiredFixed")
-                        .schema(
-                            InternalSchema.builder()
-                                .name("fixed")
-                                .dataType(InternalType.FIXED)
-                                .isNullable(false)
-                                .build())
-                        .build(),
-                    InternalField.builder()
-                        .name("optionalFixed")
-                        .schema(
-                            InternalSchema.builder()
-                                .name("fixed")
-                                .dataType(InternalType.FIXED)
-                                .isNullable(true)
-                                .build())
-                        .defaultValue(InternalField.Constants.NULL_DEFAULT_VALUE)
-                        .build()))
-            .build();
-
     InternalSchema internalSchemaAfterRoundTrip =
         InternalSchema.builder()
             .name("struct")
@@ -296,6 +266,7 @@ public class TestDeltaSchemaExtractor {
                                 .name("binary")
                                 .dataType(InternalType.BYTES)
                                 .isNullable(false)
+                                .comment("comment")
                                 .build())
                         .build(),
                     InternalField.builder()
@@ -311,12 +282,9 @@ public class TestDeltaSchemaExtractor {
             .build();
     StructType structRepresentation =
         new StructType()
-            .add("requiredFixed", DataTypes.BinaryType, false)
+            .add("requiredFixed", DataTypes.BinaryType, false, "comment")
             .add("optionalFixed", DataTypes.BinaryType, true);
 
-    Assertions.assertEquals(
-        structRepresentation,
-        DeltaSchemaExtractor.getInstance().fromInternalSchema(internalSchemaOriginal));
     Assertions.assertEquals(
         internalSchemaAfterRoundTrip,
         DeltaSchemaExtractor.getInstance().toInternalSchema(structRepresentation));
@@ -354,32 +322,25 @@ public class TestDeltaSchemaExtractor {
                                 .metadata(metadata)
                                 .build())
                         .defaultValue(InternalField.Constants.NULL_DEFAULT_VALUE)
-                        .build()))
-            .build();
-
-    InternalSchema internalSchemaTimestampNtz =
-        InternalSchema.builder()
-            .name("struct")
-            .dataType(InternalType.RECORD)
-            .isNullable(false)
-            .fields(
-                Arrays.asList(
+                        .build(),
                     InternalField.builder()
                         .name("requiredTimestampNtz")
                         .schema(
                             InternalSchema.builder()
-                                .name("timestampNtz")
+                                .name("timestamp_ntz")
                                 .dataType(InternalType.TIMESTAMP_NTZ)
                                 .isNullable(false)
+                                .metadata(metadata)
                                 .build())
                         .build(),
                     InternalField.builder()
                         .name("optionalTimestampNtz")
                         .schema(
                             InternalSchema.builder()
-                                .name("timestampNtz")
+                                .name("timestamp_ntz")
                                 .dataType(InternalType.TIMESTAMP_NTZ)
                                 .isNullable(true)
+                                .metadata(metadata)
                                 .build())
                         .defaultValue(InternalField.Constants.NULL_DEFAULT_VALUE)
                         .build()))
@@ -388,70 +349,13 @@ public class TestDeltaSchemaExtractor {
     StructType structRepresentationTimestamp =
         new StructType()
             .add("requiredTimestamp", DataTypes.TimestampType, false)
-            .add("optionalTimestamp", DataTypes.TimestampType, true);
+            .add("optionalTimestamp", DataTypes.TimestampType, true)
+            .add("requiredTimestampNtz", DataTypes.TimestampNTZType, false)
+            .add("optionalTimestampNtz", DataTypes.TimestampNTZType, true);
 
-    StructType structRepresentationTimestampNtz =
-        new StructType()
-            .add("requiredTimestampNtz", DataTypes.LongType, false)
-            .add("optionalTimestampNtz", DataTypes.LongType, true);
-
-    Assertions.assertEquals(
-        structRepresentationTimestamp,
-        DeltaSchemaExtractor.getInstance().fromInternalSchema(internalSchemaTimestamp));
     Assertions.assertEquals(
         internalSchemaTimestamp,
         DeltaSchemaExtractor.getInstance().toInternalSchema(structRepresentationTimestamp));
-    Assertions.assertEquals(
-        structRepresentationTimestampNtz,
-        DeltaSchemaExtractor.getInstance().fromInternalSchema(internalSchemaTimestampNtz));
-  }
-
-  @Test
-  public void testEnums() {
-    Map<InternalSchema.MetadataKey, Object> requiredEnumMetadata = new HashMap<>();
-    requiredEnumMetadata.put(InternalSchema.MetadataKey.ENUM_VALUES, Arrays.asList("ONE", "TWO"));
-    Map<InternalSchema.MetadataKey, Object> optionalEnumMetadata = new HashMap<>();
-    optionalEnumMetadata.put(
-        InternalSchema.MetadataKey.ENUM_VALUES, Arrays.asList("THREE", "FOUR"));
-
-    InternalSchema internalSchema =
-        InternalSchema.builder()
-            .name("struct")
-            .dataType(InternalType.RECORD)
-            .isNullable(false)
-            .fields(
-                Arrays.asList(
-                    InternalField.builder()
-                        .name("requiredEnum")
-                        .schema(
-                            InternalSchema.builder()
-                                .name("REQUIRED_ENUM")
-                                .dataType(InternalType.ENUM)
-                                .isNullable(false)
-                                .metadata(requiredEnumMetadata)
-                                .build())
-                        .build(),
-                    InternalField.builder()
-                        .name("optionalEnum")
-                        .schema(
-                            InternalSchema.builder()
-                                .name("OPTIONAL_ENUM")
-                                .dataType(InternalType.ENUM)
-                                .isNullable(true)
-                                .metadata(optionalEnumMetadata)
-                                .build())
-                        .defaultValue(InternalField.Constants.NULL_DEFAULT_VALUE)
-                        .build()))
-            .build();
-
-    StructType structRepresentation =
-        new StructType()
-            .add("requiredEnum", DataTypes.StringType, false)
-            .add("optionalEnum", DataTypes.StringType, true);
-
-    Assertions.assertEquals(
-        structRepresentation,
-        DeltaSchemaExtractor.getInstance().fromInternalSchema(internalSchema));
   }
 
   @Test
@@ -564,9 +468,6 @@ public class TestDeltaSchemaExtractor {
                 false)
             .add("recordMap", DataTypes.createMapType(DataTypes.IntegerType, mapElement, true));
 
-    Assertions.assertEquals(
-        structRepresentation,
-        DeltaSchemaExtractor.getInstance().fromInternalSchema(internalSchema));
     Assertions.assertEquals(
         internalSchema, DeltaSchemaExtractor.getInstance().toInternalSchema(structRepresentation));
   }
@@ -657,9 +558,6 @@ public class TestDeltaSchemaExtractor {
             .add("intList", DataTypes.createArrayType(DataTypes.IntegerType, false), false)
             .add("recordList", DataTypes.createArrayType(elementSchema, true), true);
 
-    Assertions.assertEquals(
-        structRepresentation,
-        DeltaSchemaExtractor.getInstance().fromInternalSchema(internalSchema));
     Assertions.assertEquals(
         internalSchema, DeltaSchemaExtractor.getInstance().toInternalSchema(structRepresentation));
   }
@@ -681,6 +579,7 @@ public class TestDeltaSchemaExtractor {
                                 .name("struct")
                                 .dataType(InternalType.RECORD)
                                 .isNullable(true)
+                                .comment("comment")
                                 .fields(
                                     Arrays.asList(
                                         InternalField.builder()
@@ -691,6 +590,7 @@ public class TestDeltaSchemaExtractor {
                                                     .name("integer")
                                                     .dataType(InternalType.INT)
                                                     .isNullable(true)
+                                                    .comment("nestedOptionalIntComment")
                                                     .build())
                                             .defaultValue(
                                                 InternalField.Constants.NULL_DEFAULT_VALUE)
@@ -740,16 +640,18 @@ public class TestDeltaSchemaExtractor {
             .add(
                 "nestedOne",
                 new StructType()
-                    .add("nestedOptionalInt", DataTypes.IntegerType, true)
+                    .add(
+                        "nestedOptionalInt",
+                        DataTypes.IntegerType,
+                        true,
+                        "nestedOptionalIntComment")
                     .add("nestedRequiredDouble", DataTypes.DoubleType, false)
                     .add(
                         "nestedTwo",
                         new StructType().add("doublyNestedString", DataTypes.StringType, true),
                         false),
-                true);
-    Assertions.assertEquals(
-        structRepresentation,
-        DeltaSchemaExtractor.getInstance().fromInternalSchema(internalSchema));
+                true,
+                "comment");
     Assertions.assertEquals(
         internalSchema, DeltaSchemaExtractor.getInstance().toInternalSchema(structRepresentation));
   }
@@ -929,9 +831,6 @@ public class TestDeltaSchemaExtractor {
                         .defaultValue(InternalField.Constants.NULL_DEFAULT_VALUE)
                         .build()))
             .build();
-    Assertions.assertEquals(
-        structRepresentation,
-        DeltaSchemaExtractor.getInstance().fromInternalSchema(internalSchema));
     Assertions.assertEquals(
         internalSchema, DeltaSchemaExtractor.getInstance().toInternalSchema(structRepresentation));
   }
