@@ -28,6 +28,7 @@ import java.io.IOException;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.UUID;
@@ -59,9 +60,9 @@ import org.apache.xtable.model.schema.PartitionTransformType;
 import org.apache.xtable.model.stat.ColumnStat;
 import org.apache.xtable.model.stat.PartitionValue;
 import org.apache.xtable.model.stat.Range;
-import org.apache.xtable.model.storage.DataFilesDiff;
 import org.apache.xtable.model.storage.FileFormat;
 import org.apache.xtable.model.storage.InternalDataFile;
+import org.apache.xtable.model.storage.InternalFilesDiff;
 import org.apache.xtable.model.storage.PartitionFileGroup;
 import org.apache.xtable.testutil.ColumnStatMapUtil;
 
@@ -119,8 +120,8 @@ public class TestBaseFileUpdatesExtractor {
             String.format("%s/%s/%s", tableBasePath, partitionPath2, fileName5),
             Collections.emptyList());
 
-    DataFilesDiff diff =
-        DataFilesDiff.builder()
+    InternalFilesDiff diff =
+        InternalFilesDiff.builder()
             .filesAdded(Arrays.asList(addedFile1, addedFile2))
             .filesRemoved(Arrays.asList(removedFile1, removedFile2, removedFile3))
             .build();
@@ -135,7 +136,7 @@ public class TestBaseFileUpdatesExtractor {
     expectedPartitionToReplacedFileIds.put(
         partitionPath1, Arrays.asList(fileName3, fileIdForFile4));
     expectedPartitionToReplacedFileIds.put(partitionPath2, Collections.singletonList(fileName5));
-    assertEquals(
+    assertEqualsIgnoreOrder(
         expectedPartitionToReplacedFileIds, replaceMetadata.getPartitionToReplacedFileIds());
 
     // validate added files
@@ -146,6 +147,16 @@ public class TestBaseFileUpdatesExtractor {
     assertWriteStatusesEquivalent(expectedWriteStatuses, replaceMetadata.getWriteStatuses());
   }
 
+  private void assertEqualsIgnoreOrder(
+      Map<String, List<String>> expected, Map<String, List<String>> actual) {
+    for (Map.Entry<String, List<String>> entry : expected.entrySet()) {
+      List<?> expectedList = entry.getValue();
+      List<?> actualList = actual.get(entry.getKey());
+      assertEquals(expectedList.size(), actualList.size());
+      assertEquals(new HashSet<>(expectedList), new HashSet<>(actualList));
+    }
+  }
+
   @Test
   void extractSnapshotChanges_emptyTargetTable() throws IOException {
     String tableBasePath = tempDir.resolve(UUID.randomUUID().toString()).toString();
