@@ -227,8 +227,8 @@ public class TestIcebergSync {
     InternalDataFile dataFile1 = getDataFile(1, Collections.emptyList());
     InternalDataFile dataFile2 = getDataFile(2, Collections.emptyList());
     InternalDataFile dataFile3 = getDataFile(3, Collections.emptyList());
-    InternalSnapshot snapshot1 = buildSnapshot(table1, dataFile1, dataFile2);
-    InternalSnapshot snapshot2 = buildSnapshot(table2, dataFile2, dataFile3);
+    InternalSnapshot snapshot1 = buildSnapshot(table1, "0", dataFile1, dataFile2);
+    InternalSnapshot snapshot2 = buildSnapshot(table2, "1", dataFile2, dataFile3);
     when(mockSchemaExtractor.toIceberg(internalSchema)).thenReturn(icebergSchema);
     when(mockSchemaExtractor.toIceberg(schema2)).thenReturn(icebergSchema2);
     ArgumentCaptor<Schema> partitionSpecSchemaArgumentCaptor =
@@ -322,9 +322,9 @@ public class TestIcebergSync {
     InternalDataFile dataFile2 = getDataFile(2, Collections.emptyList());
     InternalDataFile dataFile3 = getDataFile(3, Collections.emptyList());
     InternalDataFile dataFile4 = getDataFile(4, Collections.emptyList());
-    InternalSnapshot snapshot1 = buildSnapshot(table1, dataFile1, dataFile2);
-    InternalSnapshot snapshot2 = buildSnapshot(table2, dataFile2, dataFile3);
-    InternalSnapshot snapshot3 = buildSnapshot(table2, dataFile3, dataFile4);
+    InternalSnapshot snapshot1 = buildSnapshot(table1, "0", dataFile1, dataFile2);
+    InternalSnapshot snapshot2 = buildSnapshot(table2, "1", dataFile2, dataFile3);
+    InternalSnapshot snapshot3 = buildSnapshot(table2, "2", dataFile3, dataFile4);
     when(mockSchemaExtractor.toIceberg(internalSchema)).thenReturn(icebergSchema);
     when(mockSchemaExtractor.toIceberg(schema2)).thenReturn(icebergSchema2);
     ArgumentCaptor<Schema> partitionSpecSchemaArgumentCaptor =
@@ -398,7 +398,7 @@ public class TestIcebergSync {
     InternalDataFile dataFile1 = getDataFile(1, partitionValues1);
     InternalDataFile dataFile2 = getDataFile(2, partitionValues1);
     InternalDataFile dataFile3 = getDataFile(3, partitionValues2);
-    InternalSnapshot snapshot = buildSnapshot(table, dataFile1, dataFile2, dataFile3);
+    InternalSnapshot snapshot = buildSnapshot(table, "0", dataFile1, dataFile2, dataFile3);
 
     when(mockSchemaExtractor.toIceberg(internalSchema))
         .thenReturn(icebergSchema)
@@ -461,7 +461,7 @@ public class TestIcebergSync {
     InternalDataFile dataFile1 = getDataFile(1, partitionValues1);
     InternalDataFile dataFile2 = getDataFile(2, partitionValues1);
     InternalDataFile dataFile3 = getDataFile(3, partitionValues2);
-    InternalSnapshot snapshot = buildSnapshot(table, dataFile1, dataFile2, dataFile3);
+    InternalSnapshot snapshot = buildSnapshot(table, "0", dataFile1, dataFile2, dataFile3);
 
     when(mockSchemaExtractor.toIceberg(internalSchema)).thenReturn(icebergSchema);
     PartitionSpec partitionSpec =
@@ -515,7 +515,7 @@ public class TestIcebergSync {
     InternalDataFile dataFile1 = getDataFile(1, partitionValues1);
     InternalDataFile dataFile2 = getDataFile(2, partitionValues1);
     InternalDataFile dataFile3 = getDataFile(3, partitionValues2);
-    InternalSnapshot snapshot = buildSnapshot(table, dataFile1, dataFile2, dataFile3);
+    InternalSnapshot snapshot = buildSnapshot(table, "0", dataFile1, dataFile2, dataFile3);
 
     when(mockSchemaExtractor.toIceberg(internalSchema)).thenReturn(icebergSchema);
     PartitionSpec partitionSpec =
@@ -590,7 +590,7 @@ public class TestIcebergSync {
     InternalDataFile dataFile1 = getDataFile(1, partitionValues1);
     InternalDataFile dataFile2 = getDataFile(2, partitionValues2);
     InternalDataFile dataFile3 = getDataFile(3, partitionValues3);
-    InternalSnapshot snapshot = buildSnapshot(table, dataFile1, dataFile2, dataFile3);
+    InternalSnapshot snapshot = buildSnapshot(table, "0", dataFile1, dataFile2, dataFile3);
 
     when(mockSchemaExtractor.toIceberg(internalSchema)).thenReturn(icebergSchema);
     PartitionSpec partitionSpec =
@@ -656,7 +656,7 @@ public class TestIcebergSync {
     InternalDataFile dataFile1 = getDataFile(1, partitionValues1);
     InternalDataFile dataFile2 = getDataFile(2, partitionValues1);
     InternalDataFile dataFile3 = getDataFile(3, partitionValues2);
-    InternalSnapshot snapshot = buildSnapshot(table, dataFile1, dataFile2, dataFile3);
+    InternalSnapshot snapshot = buildSnapshot(table, "0", dataFile1, dataFile2, dataFile3);
 
     when(mockSchemaExtractor.toIceberg(internalSchema)).thenReturn(icebergSchema);
     PartitionSpec partitionSpec =
@@ -681,10 +681,111 @@ public class TestIcebergSync {
         Expressions.equal(partitionField.getSourceField().getPath(), "value1"));
   }
 
-  private InternalSnapshot buildSnapshot(InternalTable table, InternalDataFile... dataFiles) {
+  @Test
+  public void testSourceTargetMapping() throws Exception {
+    // Prepare schemas
+    List<InternalField> fields2 = new ArrayList<>(internalSchema.getFields());
+    fields2.add(
+        InternalField.builder()
+            .name("long_field")
+            .schema(InternalSchema.builder().name("long").dataType(InternalType.LONG).build())
+            .build());
+    InternalSchema schema2 = internalSchema.toBuilder().fields(fields2).build();
+    List<Types.NestedField> fields = new ArrayList<>(icebergSchema.columns());
+    fields.add(Types.NestedField.of(6, false, "long_field", Types.LongType.get()));
+    Schema icebergSchema2 = new Schema(fields);
+
+    InternalTable table1 =
+        getInternalTable(tableName, basePath, internalSchema, null, LAST_COMMIT_TIME);
+    InternalTable table2 = getInternalTable(tableName, basePath, schema2, null, LAST_COMMIT_TIME);
+
+    // Create data files and snapshots
+    InternalDataFile dataFile1 = getDataFile(1, Collections.emptyList());
+    InternalDataFile dataFile2 = getDataFile(2, Collections.emptyList());
+    InternalDataFile dataFile3 = getDataFile(3, Collections.emptyList());
+    InternalSnapshot snapshot1 = buildSnapshot(table1, "0", dataFile1, dataFile2);
+    InternalSnapshot snapshot2 = buildSnapshot(table2, "1", dataFile2, dataFile3);
+
+    when(mockSchemaExtractor.toIceberg(internalSchema)).thenReturn(icebergSchema);
+    when(mockSchemaExtractor.toIceberg(schema2)).thenReturn(icebergSchema2);
+    when(mockPartitionSpecExtractor.toIceberg(eq(null), any()))
+        .thenReturn(PartitionSpec.unpartitioned());
+
+    mockColStatsForFile(dataFile1, 2);
+    mockColStatsForFile(dataFile2, 2);
+    mockColStatsForFile(dataFile3, 1);
+
+    // Sync first snapshot
+    TableFormatSync.getInstance()
+        .syncSnapshot(Collections.singletonList(conversionTarget), snapshot1);
+    Optional<String> targetIdentifier1 =
+        conversionTarget.getTargetCommitIdentifier(snapshot1.getSourceIdentifier());
+    assertTrue(targetIdentifier1.isPresent());
+    assertEquals(
+        targetIdentifier1.get(), String.valueOf(getTable(basePath).currentSnapshot().snapshotId()));
+
+    // Sync second snapshot
+    TableFormatSync.getInstance()
+        .syncSnapshot(Collections.singletonList(conversionTarget), snapshot2);
+    Optional<String> targetIdentifier2 =
+        conversionTarget.getTargetCommitIdentifier(snapshot2.getSourceIdentifier());
+    assertTrue(targetIdentifier2.isPresent());
+    assertEquals(
+        targetIdentifier2.get(), String.valueOf(getTable(basePath).currentSnapshot().snapshotId()));
+
+    // Validate that an unknown source ID returns empty
+    Optional<String> emptyTargetIdentifier = conversionTarget.getTargetCommitIdentifier("999");
+    assertFalse(emptyTargetIdentifier.isPresent());
+  }
+
+  @Test
+  public void testGetTargetCommitIdentifierWithNullSourceIdentifier() {
+    // Prepare schemas
+    List<InternalField> fields2 = new ArrayList<>(internalSchema.getFields());
+    fields2.add(
+        InternalField.builder()
+            .name("long_field")
+            .schema(InternalSchema.builder().name("long").dataType(InternalType.LONG).build())
+            .build());
+    InternalSchema schema2 = internalSchema.toBuilder().fields(fields2).build();
+    List<Types.NestedField> fields = new ArrayList<>(icebergSchema.columns());
+    fields.add(Types.NestedField.of(6, false, "long_field", Types.LongType.get()));
+    Schema icebergSchema2 = new Schema(fields);
+
+    InternalTable internalTable =
+        getInternalTable(tableName, basePath, internalSchema, null, LAST_COMMIT_TIME);
+
+    // Create data files and snapshots
+    InternalDataFile dataFile1 = getDataFile(1, Collections.emptyList());
+    InternalSnapshot snapshot = buildSnapshot(internalTable, "0", dataFile1);
+    when(mockSchemaExtractor.toIceberg(internalSchema)).thenReturn(icebergSchema);
+    when(mockSchemaExtractor.toIceberg(schema2)).thenReturn(icebergSchema2);
+    when(mockPartitionSpecExtractor.toIceberg(eq(null), any()))
+        .thenReturn(PartitionSpec.unpartitioned());
+    mockColStatsForFile(dataFile1, 2);
+
+    // Mock the snapshot sync process like getSyncResult()
+    conversionTarget.beginSync(internalTable);
+    TableSyncMetadata tableSyncMetadata =
+        TableSyncMetadata.of(internalTable.getLatestCommitTime(), snapshot.getPendingCommits());
+    conversionTarget.syncMetadata(tableSyncMetadata);
+    conversionTarget.syncSchema(internalTable.getReadSchema());
+    conversionTarget.syncPartitionSpec(internalTable.getPartitioningFields());
+    conversionTarget.syncFilesForSnapshot(snapshot.getPartitionedDataFiles());
+    conversionTarget.completeSync();
+
+    // No crash should happen during the process
+    Optional<String> targetIdentifier = conversionTarget.getTargetCommitIdentifier("0");
+    // The targetIdentifier is expected to not be found
+    assertFalse(targetIdentifier.isPresent());
+  }
+
+  private InternalSnapshot buildSnapshot(
+      InternalTable table, String sourceIdentifier, InternalDataFile... dataFiles) {
     return InternalSnapshot.builder()
         .table(table)
         .partitionedDataFiles(PartitionFileGroup.fromFiles(Arrays.asList(dataFiles)))
+        .sourceIdentifier(sourceIdentifier)
         .build();
   }
 
