@@ -45,6 +45,7 @@ import org.apache.xtable.hudi.HudiConversionSourceProvider;
 import org.apache.xtable.model.schema.PartitionTransformType;
 import org.apache.xtable.model.sync.SyncMode;
 import org.apache.xtable.model.sync.SyncResult;
+import org.apache.xtable.model.sync.SyncStatusCode;
 
 /**
  * A HoodieSyncTool for syncing a Hudi table to other formats (Delta and Iceberg) with
@@ -106,8 +107,8 @@ public class XTableSyncTool extends HoodieSyncTool {
         results.entrySet().stream()
             .filter(
                 entry ->
-                    entry.getValue().getStatus().getStatusCode()
-                        != SyncResult.SyncStatusCode.SUCCESS)
+                    entry.getValue().getTableFormatSyncStatus().getStatusCode()
+                        != SyncStatusCode.SUCCESS)
             .map(Map.Entry::getKey)
             .collect(Collectors.joining(","));
     if (!failingFormats.isEmpty()) {
